-- Various tests to perform with hrtime.e
--
-- Times are stored in sequence and only printed after timings to
-- prevent printing from interfering with timings
--
-- NOTE: ANY argument to this program will cause HLT testing to
--  be bypassed.
--
-- When executing under plain DOS, use:
--
--      ex testhrt.ex anything > file.txt
--
--
-- This program is designed for output to be redirected (piped) to
-- a file
--
--


include hrtime.e
include get.e
include machine.e  -- for set_rand
include dll.e -- for free_console()

puts(1,"\n")

-- ###################################################################
--
-- Check overhead time for successive calls to hrtime()
--
-- ###################################################################


constant NOVHDS = 10
sequence ovhds
ovhds = repeat(0.0,NOVHDS)

atom t1, t2

for i = 1 to NOVHDS do
    t1 = hrtime()
    t2 = hrtime()
    ovhds[i] = t2 - t1
end for

puts(1,"\nMeasure hrtime() overhead (the time between successive\n" &
         "  calls to hrtime().)\n\n")

for i = 1 to NOVHDS do
    printf(1,"hrtime() overhead is %10.3e seconds.\n",ovhds[i])
end for

-- ###################################################################
--
-- Check repeatability of measuring CPU speed
--
-- ###################################################################

sequence sticks, cpuspeeds

sticks = {1,2,3,4,5,10,20,30,50,75}
cpuspeeds = repeat(0.0,length(sticks))


puts(1, "\nCheck repeatability of measuring CPU speed.\n" &
        "  Under DOS32 I use tick_rate(997) while measuring cpu speed.\n" &
        "    Win32 has a tick rate of 100.\n" &
        "    Linux has a tick rate of 1000.\n" &
        "    At least according to Eu doc.\n\n")

for i = 1 to length(sticks) do
    cpuspeeds[i] = measure_cpu_speed(sticks[i])
end for

for i = 1 to length(sticks) do
    printf(1,"Cpu Speed = %10.2fMHz for %5d ticks.\n",
        {cpuspeeds[i], sticks[i]})
end for

-- ###################################################################
--
-- Measure rate of standard ticks
--
-- ###################################################################

sequence stdtick

integer numticks
numticks = 10

atom dum

stdtick = repeat(0.0, numticks)

dum = time()
while dum = time() do end while -- sync to tick

for i = 1 to numticks do
    stdtick[i] = hrtime()
    dum = time()
    while dum = time() do end while -- sync to tick
    stdtick[i] = hrtime() - stdtick[i]
end for

puts(1,"\nMeasure rate of standard ticks.\n\n")

for i = 1 to numticks do
    printf(1,"Std tick rate is %8.2f times/sec.\n", {1.0/stdtick[i]})
end for

-- ###################################################################
--
-- Check to see if time() gives control to OS
-- by counting number of times through "while"
--
-- ###################################################################

atom cntr, elptime
cntr = 0

sequence works
works = repeat({0.0,0.0},numticks)

dum = time()
while dum = time() do end while -- sync to tick

for i = 1 to numticks do
    elptime = hrtime()
    cntr = 0.0
    dum = time()
    while dum = time() do cntr += 1.0 end while -- wait for tick
    elptime = hrtime() - elptime
    works[i][1] = cntr
    works[i][2] = elptime
end for

puts(1,"\nMeasure amount of work done while waiting for tick.\n\n")

for i = 1 to numticks do
    printf(1,"Was able to incr cnt %10d times while waiting for %.2e secs.\n",
            {works[i][1], works[i][2]})
end for

-- ###################################################################
--
-- Check speed of floating point operations
-- Add up inverse of sqrt of 100000 random numbers 10 times
--
-- ###################################################################

sequence ftimes, fnums, fsqrts

constant NUM = 100000
constant TIMES = 10

atom sum
sum = 0.0

puts(1,"\nExplore speed of floating point operations.\n")

printf(1,"  fnums is sequence of %d random floating point numbers.\n\n",
         {NUM})

set_rand(314159) -- use repeatable sequence of random numbers

ftimes = repeat(0.0,TIMES)

fnums = repeat(0.0,NUM)
fsqrts = repeat(0.0,NUM)

for i = 1 to NUM do
    fnums[i] = 1.00001 * rand(10000)
end for

for i = 1 to TIMES do
    sum = 0.0
    ftimes[i] = hrtime()
    for j = 1 to NUM do
        sum += 1.0/sqrt(fnums[j])
    end for
    ftimes[i] = hrtime() - ftimes[i]
end for

for i = 1 to TIMES do
    printf(1,"     Computing sum += 1.0/sqrt(fnums[i]) takes %.2e secs.\n",
            {ftimes[i]})
end for


-- ###################################################################
--
-- Experiment with call to "HLT"
-- Time delay after calling HLT
--
-- ###################################################################

-- Check to see if command line has 1 or more arguments
-- If so, assume that user is running under straight DOS and do not
-- perform HLT experiments

if length(command_line()) >= 3 then 
    abort(0)    
end if

puts(1,"\nBegin testing use of HLT.\n" &
       "  Note: This will cause a program crash if run under straight DOS,\n" &
       "        but should be OK in a Windows DOS box or as a Windows\n" &
       "        cmdline program or in Linux (or FreeBSD).\n" &
       "  call(HLT) will return control to OS until OS gives it back.\n\n" )

for i = 1 to numticks do
    dum = hrtime()
    call(HLT)
    stdtick[i] = hrtime() - dum
end for

for i = 1 to numticks do
    printf(1,"Time delay after calling HLT is %.2e seconds.\n",
        {stdtick[i]})
end for

-- ###################################################################
--
-- Check ability to set value for CPU speed
--
-- ###################################################################


atom old_speed, faux_speed
old_speed = 0.0

old_speed = get_cpu_speed()
faux_speed = old_speed/100.0
old_speed = set_cpu_speed(faux_speed)

printf(1,"\nCheck ability to set value for CPU speed.\n" &
         "\n  Very useful if measured speed is too variable.\n" &
         "\n  Measured CPU Speed is %.2f MHz, but I set it to %.2f MHz.\n",
        {old_speed, faux_speed})

printf(1,"  hrtime() should now report that HLT delay is about %.2f times > than\n" &
         "  above, but the actual delay time has not changed.\n\n",
        {old_speed/faux_speed})        

    -- Experiment with call to "HLT"
    -- Time delay after calling HLT
    

for i = 1 to numticks do
    dum = hrtime()
    call(HLT)
    stdtick[i] = hrtime() - dum
end for

for i = 1 to numticks do
    printf(1,"Time delay after calling HLT is %.2e seconds.\n",
        {stdtick[i]})
end for





--
--
-- Note: this program is designed to have output redirected (piped)
--  to a file, thus I get rid of console immediately!
--
--

if platform() != DOS32 then 
    free_console()
end if



