-- Some calcs relating to use of the 64-bit TimeStampCounter on
--      Intel (and compatible) Pentium+ CPUs and compatibles
--          Includes Pentium, Pentium MMX, Pentium Pro,
--              Pentium II, III, 4, AMD Athlons+
--
-- Currently, the "tick" rate of the TSC is the frequency of the
--      internal CPU clock (bus speed * CPU multiplier)
--
-- A Double float (float_64) has 53 bits of resolution
--      MANTISSA is 52 bits, but the 53rd bit is assumed to be 1
--      (Normalized mantissa)
--
-- Intel guarantees (in its IA32 family Pentium+ architecture
--      manuals) that the TSC will not overlow for "at least"
--      5 yrs.
--
-- Speculation:
--      If the CPU speeds increase to the point that the guarantee would
--          not be true, then the "tick" rate would be reduced, probably
--          by dividing by some integral n.
--      Of course, we're talking CPU Speed > 100GHz for this to happen.
--      By then even cell phones might be native 64-bit, with 128-bit
--          integer support in compilers, so, I don't really care.
--
-- Note: the TSC is set to 0 upon power-up or reset, so this overflow
--      time is for continuous operation!
--
-- Let's explore that fact for various CPU speeds.

constant GHz = 1.0e+9

sequence speeds     -- in GHz
speeds = {.001,.010,.100,.500,1.000,2.000,3.000,4.000,
          5.000,10.000,100.000,1000.000}
sequence speed
speed = repeat(0.0,3)
sequence wkstring 

sequence ydhs
    ydhs = {{},{}}

sequence code_ydhs
code_ydhs = {0, 365, 24, 60, 60}    
sequence code_ydhs_labels
sequence code_ydhs_formats
code_ydhs_labels = {"y ", "d ", "h ", "m ", "s"}
code_ydhs_formats = {"%6d","%3d","%2d", "%2d", "%2d"}

function encode(atom a, sequence code)
-- decode value a into sequence
-- result is a sequence equal in length to code
-- 

integer num_of

integer sig_terms

    if atom(code[1]) then
        sig_terms = length(code)
    else
        sig_terms = code[2]
        code = code[1]              -- drop sig_terms from code
    end if

    if sig_terms < 1 then 
        sig_terms = 1
    elsif sig_terms > length(code) then
        sig_terms = length(code)
    end if

-- Prepare code
    for i = length(code) - 1 to 1 by -1 do
        code[i] *= code[i+1]
    end for

--    ? code

--  Encode atom a to specified number of significant terms

    num_of = 0  -- count of significant terms

    for i = 1 to length(code) - 1 do

        if a > code[i+1] or num_of then   -- Got a significant term
            code[i] = floor(a/code[i+1])
            a -= code[i]*code[i+1]
            num_of += 1
        else
            code[i] = 0
        end if

        if num_of = sig_terms then
            code[i] = a
            a = 0
        end if

    end for

    code[length(code)] = a

--    ? code

    return code

end function

atom p2_64
p2_64 = power(2,64) - 1

atom p2_53
p2_53 = power(2,53) - 1

--? encode(p2_53,code_ydhs) -- can a 64-bit float handle computation
                            -- to the nearest second? Yes!

for i = 1 to length(speeds) do
    speed[1] = speeds[i]
    -- secs to overflow 64 bits
    speed[2] = p2_64/(speed[1]*GHz)
    speed[3] = p2_53/(speed[1]*GHz)
    speeds[i] = speed
end for

--? speeds
puts(1,
 "CPU Speed   64 bit TSC Overflows in: 64-bit Float loses precision:\n")
puts(1,
 "----------- ------------------------ ------------------------\n")
--1000.000GHz      0y 213d 12h  5m 44s      0y   0d  2h 30m  7s 

for i= 1 to length(speeds) do
    wkstring = {}
    wkstring &= sprintf("%8.3fGHz ",speeds[i][1])
    for j = 2 to 3 do
        ydhs[j-1] = encode(speeds[i][j],code_ydhs)
    end for
--    ? ydhs
    for j = 1 to 2 do
        for k = 1 to length(code_ydhs) do
            wkstring &= sprintf(code_ydhs_formats[k],ydhs[j][k])
                        & code_ydhs_labels[k]
        end for
        wkstring &= " "
    end for
    puts(1,wkstring&"\n")
end for

puts(1,"\n\n(y,d,h,m,s) suffix means (years,days,hours,minutes,seconds.)" &
       "\n  where a year is assumed to have exactly 365 days (so sue me!)")

-- To do:
--      Test Significant Terms option
--      Bullet proof encode
--      Handle atomic code
--      handle single dimensional code formats
-- Done:
--      How to prevent things like 60 seconds instead of 59.9999999
--          i.e. force truncation?
--          Solved! Just use %d which truncates to integer, rather than
--              %f which rounds.

